# -*- coding: utf-8 -*-
#!/usr/bin/python

import f7

import json
import threading
import traceback
import tornado.web
import tornado.ioloop

class _mainHandler(tornado.web.RequestHandler):

    def get(self):
        instance()._dispatchRequest(self)

_IM_DoneCallback = 100

class _App:
    _instance = None

    @classmethod
    def instance(cls):
        if not cls._instance:
            cls._instance = _App()
        return cls._instance

    def __init__(self):
        self._listened = False
        self._requestHandler = {}
        self._app = tornado.web.Application([
            (r"/webapp/index[\.]php", _mainHandler),
        ])
        self.registerHandler('Ops', 'selfChecking', self.__selfChecking)
        self._immsgLock = threading.Lock()
        self._immsgList = []
        tornado.ioloop.PeriodicCallback(self._processIMMsg, 100).start()

    def init(self, log_dir):
        f7.udplog.init()
        f7.udplog.setLogDirAndCreate(log_dir)

    def start(self):
        if self._listened:
            tornado.ioloop.IOLoop.current().start()
        f7.timer.unInit()
        f7.udplog.unInit()

    def listen(self, port):
        self._app.listen(port)
        self._listened = True

    def callAt(self, when, callback):
        tornado.ioloop.IOLoop.current().call_at(when, callback)

    def callLater(self, when, callback):
        tornado.ioloop.IOLoop.current().call_later(when, callback)

    def registerHandler(self, c, a, callback):
        self._requestHandler[c + '$' + a] = callback

    def _dispatchRequest(self, request):
        c = request.get_argument('c', '')
        a = request.get_argument('a', '')
        handler = self._requestHandler.get(c + '$' + a, None)
        if not handler:
            request.write('')
            return
        try:
            response = handler(lambda param, def_val = '': request.get_argument(param, def_val))
            request.write(response)
        except Exception as e:
            f7.udplog.error('dispatch request ' + str(e))
            request.write(json.dumps({
                'errcode': 200,
                'errmsg': '服务内部错误',
            }))

    def __selfChecking(self, request):
        return json.dumps({
            'errcode': 0,
            'errmsg': '',
            'healthy': 1,
            'max_rundelay': 10
        })

    def _processIMMsg(self):
        self._immsgLock.acquire()
        try:
            if len(self._immsgList) > 0:
                for node in self._immsgList:
                    self._dispatchIMMsg(node[0], node[1])
        finally:
            self._immsgList = []
            self._immsgLock.release()

    def _dispatchIMMsg(self, msgid, callback):
        try:
            if msgid == _IM_DoneCallback:
                self._IMDoneCallback(callback)
        except Exception as e:
            f7.udplog.error('_dispatchIMMsg ' + str(e))

    def _addIMMsg(self, msgid, callback):
        self._immsgLock.acquire()
        try:
            self._immsgList.append((msgid, callback))
        finally:
            self._immsgLock.release()

    def _IMDoneCallback(self, callback):
        callback()

    def createAsyncTask(self, done_callback, func, *args):
        def thread_func(args):
            func(args)
            if done_callback:
                self._addIMMsg(_IM_DoneCallback, done_callback)
        thread = threading.Thread(target = thread_func, args = args)
        thread.start()

def instance():
    return _App.instance()
